@@ -1,37 +1,30 @@
 """
 Classes for including text in a figure.
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
+from __future__ import absolute_import, division, print_function
 
-from matplotlib.externals import six
-from matplotlib.externals.six.moves import zip
-
-import math
-import warnings
+import six
+from six.moves import zip
 
 import contextlib
+import logging
+import math
+import warnings
+import weakref
 
 import numpy as np
 
-from matplotlib import cbook
-from matplotlib import rcParams
-import matplotlib.artist as artist
-from matplotlib.artist import Artist
-from matplotlib.cbook import is_string_like, maxdict
-from matplotlib import docstring
-from matplotlib.font_manager import FontProperties
-from matplotlib.patches import FancyBboxPatch
-from matplotlib.patches import FancyArrowPatch, Rectangle
-import matplotlib.transforms as mtransforms
-from matplotlib.transforms import Affine2D, Bbox, Transform
-from matplotlib.transforms import BboxBase, BboxTransformTo
-from matplotlib.lines import Line2D
-from matplotlib.path import Path
-from matplotlib.artist import allow_rasterization
+from . import artist, cbook, docstring, rcParams
+from .artist import Artist
+from .font_manager import FontProperties
+from .lines import Line2D
+from .patches import FancyArrowPatch, FancyBboxPatch, Rectangle
+from .textpath import TextPath  # Unused, but imported by others.
+from .transforms import (
+    Affine2D, Bbox, BboxBase, BboxTransformTo, IdentityTransform, Transform)
 
-from matplotlib.backend_bases import RendererBase
-from matplotlib.textpath import TextPath
+
+_log = logging.getLogger(__name__)
 
 
 def _process_text_args(override, fontdict=None, **kwargs):
@@ -46,8 +39,7 @@ def _process_text_args(override, fontdict=None, **kwargs):
 
 @contextlib.contextmanager
 def _wrap_text(textobj):
-    """
-    Temporarily inserts newlines to the text if the wrap option is enabled.
+    """Temporarily inserts newlines to the text if the wrap option is enabled.
     """
     if textobj.get_wrap():
         old_text = textobj.get_text()
@@ -82,61 +74,6 @@ def get_rotation(rotation):
                              "None".format(rotation))
 
     return angle % 360
-# these are not available for the object inspector until after the
-# class is build so we define an initial set here for the init
-# function and they will be overridden after object defn
-docstring.interpd.update(Text="""
-    ========================== ================================================
-    Property                   Value
-    ========================== ================================================
-    alpha                      float or None
-    animated                   [True | False]
-    backgroundcolor            any matplotlib color
-    bbox                       rectangle prop dict plus key 'pad' which is a
-                               pad in points; if a boxstyle is supplied as
-                               a string, then pad is instead a fraction
-                               of the font size
-    clip_box                   a matplotlib.transform.Bbox instance
-    clip_on                    [True | False]
-    color                      any matplotlib color
-    family                     ['serif' | 'sans-serif' | 'cursive' |
-                                'fantasy' | 'monospace']
-    figure                     a matplotlib.figure.Figure instance
-    fontproperties             a matplotlib.font_manager.FontProperties
-                               instance
-    horizontalalignment or ha  ['center' | 'right' | 'left']
-    label                      any string
-    linespacing                float
-    lod                        [True | False]
-    multialignment             ['left' | 'right' | 'center' ]
-    name or fontname           string e.g.,
-                               ['Sans' | 'Courier' | 'Helvetica' ...]
-    position                   (x,y)
-    rotation                   [ angle in degrees 'vertical' | 'horizontal'
-    rotation_mode              [ None | 'anchor']
-    size or fontsize           [size in points | relative size e.g., 'smaller',
-                                                                  'x-large']
-    style or fontstyle         [ 'normal' | 'italic' | 'oblique']
-    text                       string
-    transform                  a matplotlib.transform transformation instance
-    usetex                     [True | False | None]
-    variant                    ['normal' | 'small-caps']
-    verticalalignment or va    ['center' | 'top' | 'bottom' | 'baseline']
-    visible                    [True | False]
-    weight or fontweight       ['normal' | 'bold' | 'heavy' | 'light' |
-                                'ultrabold' | 'ultralight']
-    wrap                       [True | False]
-    x                          float
-    y                          float
-    zorder                     any number
-    ========================== ===============================================
-    """)
-
-# TODO : This function may move into the Text class as a method. As a
-# matter of fact, The information from the _get_textbox function
-# should be available during the Text._get_layout() call, which is
-# called within the _get_textbox. So, it would better to move this
-# function as a method with some refactoring of _get_layout method.
 
 
 def _get_textbox(text, renderer):
@@ -145,12 +82,17 @@ def _get_textbox(text, renderer):
     :meth:`matplotlib.text.Text.get_extents` method, The bbox size of
     the text before the rotation is calculated.
     """
+    # TODO : This function may move into the Text class as a method. As a
+    # matter of fact, The information from the _get_textbox function
+    # should be available during the Text._get_layout() call, which is
+    # called within the _get_textbox. So, it would better to move this
+    # function as a method with some refactoring of _get_layout method.
 
     projected_xs = []
     projected_ys = []
 
     theta = np.deg2rad(text.get_rotation())
-    tr = mtransforms.Affine2D().rotate(-theta)
+    tr = Affine2D().rotate(-theta)
 
     _, parts, d = text._get_layout(renderer)
 
@@ -167,7 +109,7 @@ def _get_textbox(text, renderer):
     xt_box, yt_box = min(projected_xs), min(projected_ys)
     w_box, h_box = max(projected_xs) - xt_box, max(projected_ys) - yt_box
 
-    tr = mtransforms.Affine2D().rotate(theta)
+    tr = Affine2D().rotate(theta)
 
     x_box, y_box = tr.transform_point((xt_box, yt_box))
 
@@ -179,10 +121,9 @@ class Text(Artist):
     Handle storing and drawing of text in window or data coordinates.
     """
     zorder = 3
+    _cached = cbook.maxdict(50)
 
-    _cached = maxdict(50)
-
-    def __str__(self):
+    def __repr__(self):
         return "Text(%g,%g,%s)" % (self._x, self._y, repr(self._text))
 
     def __init__(self,
@@ -214,7 +155,7 @@ class Text(Artist):
             color = rcParams['text.color']
         if fontproperties is None:
             fontproperties = FontProperties()
-        elif is_string_like(fontproperties):
+        elif isinstance(fontproperties, six.string_types):
             fontproperties = FontProperties(fontproperties)
 
         self.set_text(text)
@@ -238,10 +179,12 @@ class Text(Artist):
         """
         Update properties from a dictionary.
         """
-        bbox = kwargs.pop('bbox', None)
+        # Update bbox last, as it depends on font properties.
+        sentinel = object()  # bbox can be None, so use another sentinel.
+        bbox = kwargs.pop("bbox", sentinel)
         super(Text, self).update(kwargs)
-        if bbox:
-            self.set_bbox(bbox)  # depends on font properties
+        if bbox is not sentinel:
+            self.set_bbox(bbox)
 
     def __getstate__(self):
         d = super(Text, self).__getstate__()
@@ -257,7 +200,7 @@ class Text(Artist):
 
         Returns True or False.
         """
-        if six.callable(self._contains):
+        if callable(self._contains):
             return self._contains(self, mouseevent)
 
         if not self.get_visible() or self._renderer is None:
@@ -296,11 +239,16 @@ class Text(Artist):
 
     def set_rotation_mode(self, m):
         """
-        set text rotation mode. If "anchor", the un-rotated text
-        will first aligned according to their *ha* and
-        *va*, and then will be rotated with the alignement
-        reference point as a origin. If None (default), the text will be
-        rotated first then will be aligned.
+        Set text rotation mode.
+
+        .. ACCEPTS: [ None | "default" | "anchor" ]
+
+        Parameters
+        ----------
+        m : ``None`` or ``"default"`` or ``"anchor"``
+            If ``None`` or ``"default"``, the text will be first rotated, then
+            aligned according to their horizontal and vertical alignments.  If
+            ``"anchor"``, then alignment occurs before rotation.
         """
         if m is None or m in ["anchor", "default"]:
             self._rotation_mode = m
@@ -331,7 +279,7 @@ class Text(Artist):
         multiple-alignment information. Note that it returns an extent
         of a rotated text when necessary.
         """
-        key = self.get_prop_tup()
+        key = self.get_prop_tup(renderer=renderer)
         if key in self._cached:
             return self._cached[key]
 
@@ -354,7 +302,7 @@ class Text(Artist):
 
         baseline = 0
         for i, line in enumerate(lines):
-            clean_line, ismath = self.is_math_text(line)
+            clean_line, ismath = self.is_math_text(line, self.get_usetex())
             if clean_line:
                 w, h, d = renderer.get_text_width_height_descent(clean_line,
                                                         self._fontproperties,
@@ -398,8 +346,7 @@ class Text(Artist):
 
         # the corners of the unrotated bounding box
         cornersHoriz = np.array(
-            [(xmin, ymin), (xmin, ymax), (xmax, ymax), (xmax, ymin)],
-            np.float_)
+            [(xmin, ymin), (xmin, ymax), (xmax, ymax), (xmax, ymin)], float)
         cornersHoriz[:, 1] -= descent
 
         # now rotate the bbox
@@ -436,6 +383,8 @@ class Text(Artist):
                 offsety = (ymin + height)
             elif valign == 'baseline':
                 offsety = (ymin + height) - baseline
+            elif valign == 'center_baseline':
+                offsety = ymin + height - baseline / 2.0
             else:
                 offsety = ymin
         else:
@@ -455,6 +404,8 @@ class Text(Artist):
                 offsety = ymax1
             elif valign == 'baseline':
                 offsety = ymax1 - baseline
+            elif valign == 'center_baseline':
+                offsety = (ymin1 + ymax1 - baseline) / 2.0
             else:
                 offsety = ymin1
 
@@ -502,7 +453,8 @@ class Text(Artist):
                     pad = 0.3
 
             # boxstyle could be a callable or a string
-            if is_string_like(boxstyle) and "pad" not in boxstyle:
+            if (isinstance(boxstyle, six.string_types)
+                    and "pad" not in boxstyle):
                 boxstyle += ",pad=%0.2f" % pad
 
             bbox_transmuter = props.pop("bbox_transmuter", None)
@@ -512,7 +464,7 @@ class Text(Artist):
                                     1., 1.,
                                     boxstyle=boxstyle,
                                     bbox_transmuter=bbox_transmuter,
-                                    transform=mtransforms.IdentityTransform(),
+                                    transform=IdentityTransform(),
                                     **props)
         else:
             self._bbox_patch = None
@@ -547,7 +499,7 @@ class Text(Artist):
             x_box, y_box, w_box, h_box = _get_textbox(self, renderer)
             self._bbox_patch.set_bounds(0., 0., w_box, h_box)
             theta = np.deg2rad(self.get_rotation())
-            tr = mtransforms.Affine2D().rotate(theta)
+            tr = Affine2D().rotate(theta)
             tr = tr.translate(posx + x_box, posy + y_box)
             self._bbox_patch.set_transform(tr)
             fontsize_in_pixel = renderer.points_to_pixels(self.get_size())
@@ -562,7 +514,7 @@ class Text(Artist):
         x_box, y_box, w_box, h_box = _get_textbox(self, renderer)
         self._bbox_patch.set_bounds(0., 0., w_box, h_box)
         theta = np.deg2rad(self.get_rotation())
-        tr = mtransforms.Affine2D().rotate(theta)
+        tr = Affine2D().rotate(theta)
         tr = tr.translate(posx + x_box, posy + y_box)
         self._bbox_patch.set_transform(tr)
         fontsize_in_pixel = renderer.points_to_pixels(self.get_size())
@@ -614,23 +566,28 @@ class Text(Artist):
         """
         Set whether artist uses clipping.
 
-        When False artists will be visible out side of the axes which
-        can lead to unexpected results.
+        When False, artists will be visible outside of the axes, which can lead
+        to unexpected results.
 
-        ACCEPTS: [True | False]
+        Parameters
+        ----------
+        b : bool
+            .. ACCEPTS: bool
         """
         super(Text, self).set_clip_on(b)
         self._update_clip_properties()
 
     def get_wrap(self):
-        """
-        Returns the wrapping state for the text.
-        """
+        """Returns the wrapping state for the text."""
         return self._wrap
 
     def set_wrap(self, wrap):
-        """
-        Sets the wrapping state for the text.
+        """Sets the wrapping state for the text.
+
+        Parameters
+        ----------
+        wrap : bool
+            .. ACCEPTS: bool
         """
         self._wrap = wrap
 
@@ -731,7 +688,7 @@ class Text(Artist):
 
         return wrapped_str + line
 
-    @allow_rasterization
+    @artist.allow_rasterization
     def draw(self, renderer):
         """
         Draws the :class:`Text` object to the given *renderer*.
@@ -740,7 +697,7 @@ class Text(Artist):
             self._renderer = renderer
         if not self.get_visible():
             return
-        if self.get_text().strip() == '':
+        if self.get_text() == '':
             return
 
         renderer.open_group('text', self.get_gid())
@@ -753,9 +710,10 @@ class Text(Artist):
             # position in Text, and dash position in TextWithDash:
             posx = float(textobj.convert_xunits(textobj._x))
             posy = float(textobj.convert_yunits(textobj._y))
-            if not np.isfinite(posx) or not np.isfinite(posy):
-                raise ValueError("posx and posy should be finite values")
             posx, posy = trans.transform_point((posx, posy))
+            if not np.isfinite(posx) or not np.isfinite(posy):
+                _log.warning("posx and posy should be finite values")
+                return
             canvasw, canvash = renderer.get_canvas_width_height()
 
             # draw the FancyBboxPatch
@@ -777,7 +735,8 @@ class Text(Artist):
                 y = y + posy
                 if renderer.flipy():
                     y = canvash - y
-                clean_line, ismath = textobj.is_math_text(line)
+                clean_line, ismath = textobj.is_math_text(line,
+                                                          self.get_usetex())
 
                 if textobj.get_path_effects():
                     from matplotlib.patheffects import PathEffectRenderer
@@ -881,7 +840,7 @@ class Text(Artist):
     def get_unitless_position(self):
         "Return the unitless position of the text as a tuple (*x*, *y*)"
         # This will get the position with all unit information stripped away.
-        # This is here for convienience since it is done in several locations.
+        # This is here for convenience since it is done in several locations.
         x = float(self.convert_xunits(self._x))
         y = float(self.convert_yunits(self._y))
         return x, y
@@ -892,7 +851,7 @@ class Text(Artist):
         # specified with 'set_x' and 'set_y'.
         return self._x, self._y
 
-    def get_prop_tup(self):
+    def get_prop_tup(self, renderer=None):
         """
         Return a hashable tuple of properties.
 
@@ -901,11 +860,13 @@ class Text(Artist):
         need to know if the text has changed.
         """
         x, y = self.get_unitless_position()
+        renderer = renderer or self._renderer
         return (x, y, self.get_text(), self._color,
                 self._verticalalignment, self._horizontalalignment,
                 hash(self._fontproperties),
                 self._rotation, self._rotation_mode,
-                self.figure.dpi, id(self._renderer),
+                self.figure.dpi, weakref.ref(renderer),
+                self._linespacing
                 )
 
     def get_text(self):
@@ -949,12 +910,14 @@ class Text(Artist):
         if dpi is not None:
             dpi_orig = self.figure.dpi
             self.figure.dpi = dpi
-        if self.get_text().strip() == '':
+        if self.get_text() == '':
             tx, ty = self._get_xy_display()
             return Bbox.from_bounds(tx, ty, 0, 0)
 
         if renderer is not None:
             self._renderer = renderer
+        if self._renderer is None:
+            self._renderer = self.figure._cachedRenderer
         if self._renderer is None:
             raise RuntimeError('Cannot get window extent w/o renderer')
 
@@ -1017,7 +980,7 @@ class Text(Artist):
         self.stale = True
 
     def set_ma(self, align):
-        'alias for set_verticalalignment'
+        'alias for set_multialignment'
         self.set_multialignment(align)
 
     def set_multialignment(self, align):
@@ -1207,7 +1170,7 @@ class Text(Artist):
         self.stale = True
 
     @staticmethod
-    def is_math_text(s):
+    def is_math_text(s, usetex=None):
         """
         Returns a cleaned string and a boolean flag.
         The flag indicates if the given string *s* contains any mathtext,
@@ -1217,7 +1180,9 @@ class Text(Artist):
         """
         # Did we find an even number of non-escaped dollar signs?
         # If so, treat is as math text.
-        if rcParams['text.usetex']:
+        if usetex is None:
+            usetex = rcParams['text.usetex']
+        if usetex:
             if s == ' ':
                 s = r'\ '
             return s, 'TeX'
@@ -1234,7 +1199,7 @@ class Text(Artist):
 
         ACCEPTS: a :class:`matplotlib.font_manager.FontProperties` instance
         """
-        if is_string_like(fp):
+        if isinstance(fp, six.string_types):
             fp = FontProperties(fp)
         self._fontproperties = fp.copy()
         self.stale = True
@@ -1245,23 +1210,26 @@ class Text(Artist):
 
     def set_usetex(self, usetex):
         """
-        Set this `Text` object to render using TeX (or not).
+        Parameters
+        ----------
+        usetex : bool or None
+            Whether to render using TeX, ``None`` means to use
+            :rc:`text.usetex`.
 
-        If `None` is given, the option will be reset to use the value of
-        `rcParams['text.usetex']`
+            .. ACCEPTS: bool or None
         """
         if usetex is None:
-            self._usetex = None
+            self._usetex = rcParams['text.usetex']
         else:
             self._usetex = bool(usetex)
         self.stale = True
 
     def get_usetex(self):
         """
-        Return whether this `Text` object will render using TeX.
+        Return whether this `Text` object uses TeX for rendering.
 
-        If the user has not manually set this value, it will default to
-        the value of `rcParams['text.usetex']`
+        If the user has not manually set this value, it defaults to
+        :rc:`text.usetex`.
         """
         if self._usetex is None:
             return rcParams['text.usetex']
@@ -1377,7 +1345,7 @@ class TextWithDash(Text):
     def get_unitless_position(self):
         "Return the unitless position of the text as a tuple (*x*, *y*)"
         # This will get the position with all unit information stripped away.
-        # This is here for convienience since it is done in several locations.
+        # This is here for convenience since it is done in several locations.
         x = float(self.convert_xunits(self._dashx))
         y = float(self.convert_yunits(self._dashy))
         return x, y
@@ -1388,7 +1356,7 @@ class TextWithDash(Text):
         # specified with set_x and set_y
         return self._dashx, self._dashy
 
-    def get_prop_tup(self):
+    def get_prop_tup(self, renderer=None):
         """
         Return a hashable tuple of properties.
 
@@ -1396,7 +1364,7 @@ class TextWithDash(Text):
         want to cache derived information about text (e.g., layouts) and
         need to know if the text has changed.
         """
-        props = [p for p in Text.get_prop_tup(self)]
+        props = [p for p in Text.get_prop_tup(self, renderer=renderer)]
         props.extend([self._x, self._y, self._dashlength,
                       self._dashdirection, self._dashrotation, self._dashpad,
                       self._dashpush])
@@ -1776,8 +1744,7 @@ class _AnnotationBase(object):
             tr2 = self._get_xy_transform(renderer, s2)
             tr = blended_transform_factory(tr1, tr2)
             return tr
-
-        if six.callable(s):
+        elif callable(s):
             tr = s(renderer)
             if isinstance(tr, BboxBase):
                 return BboxTransformTo(tr)
@@ -1785,14 +1752,14 @@ class _AnnotationBase(object):
                 return tr
             else:
                 raise RuntimeError("unknown return type ...")
-        if isinstance(s, Artist):
+        elif isinstance(s, Artist):
             bbox = s.get_window_extent(renderer)
             return BboxTransformTo(bbox)
         elif isinstance(s, BboxBase):
             return BboxTransformTo(s)
         elif isinstance(s, Transform):
             return s
-        elif not is_string_like(s):
+        elif not isinstance(s, six.string_types):
             raise RuntimeError("unknown coordinate type : %s" % (s,))
 
         if s == 'data':
@@ -1839,8 +1806,7 @@ class _AnnotationBase(object):
             elif unit == "fontsize":
                 fontsize = self.get_size()
                 dpp = fontsize * self.figure.get_dpi() / 72.
-                tr = Affine2D().scale(dpp,
-                                      dpp)
+                tr = Affine2D().scale(dpp, dpp)
             elif unit == "fraction":
                 w, h = bbox0.bounds[2:]
                 tr = Affine2D().scale(w, h)
@@ -1860,14 +1826,16 @@ class _AnnotationBase(object):
 
         if isinstance(self.xycoords, tuple):
             s1, s2 = self.xycoords
-            if ((is_string_like(s1) and s1.split()[0] == "offset") or
-                  (is_string_like(s2) and s2.split()[0] == "offset")):
+            if ((isinstance(s1, six.string_types)
+                 and s1.split()[0] == "offset")
+                    or (isinstance(s2, six.string_types)
+                        and s2.split()[0] == "offset")):
                 raise ValueError("xycoords should not be an offset coordinate")
             x, y = self.xy
             x1, y1 = self._get_xy(renderer, x, y, s1)
             x2, y2 = self._get_xy(renderer, x, y, s2)
             return x1, y2
-        elif (is_string_like(self.xycoords) and
+        elif (isinstance(self.xycoords, six.string_types) and
               self.xycoords.split()[0] == "offset"):
             raise ValueError("xycoords should not be an offset coordinate")
         else:
@@ -2147,7 +2115,7 @@ class Annotation(Text, _AnnotationBase):
 
         self.arrow = None
 
-        if arrowprops:
+        if arrowprops is not None:
             if "arrowstyle" in arrowprops:
                 arrowprops = self.arrowprops.copy()
                 self._arrow_relpos = arrowprops.pop("relpos", (0.5, 0.5))
@@ -2216,7 +2184,7 @@ class Annotation(Text, _AnnotationBase):
         ox0, oy0 = self._get_xy_display()
         ox1, oy1 = xy_pixel
 
-        if self.arrowprops:
+        if self.arrowprops is not None:
             x0, y0 = xy_pixel
             l, b, w, h = Text.get_window_extent(self, renderer).bounds
             r = l + w
@@ -2226,7 +2194,6 @@ class Annotation(Text, _AnnotationBase):
 
             d = self.arrowprops.copy()
             ms = d.pop("mutation_scale", self.get_size())
-            ms = renderer.points_to_pixels(ms)
             self.arrow_patch.set_mutation_scale(ms)
 
             if "arrowstyle" not in d:
@@ -2243,11 +2210,10 @@ class Annotation(Text, _AnnotationBase):
                         " use 'headlength' to set the head length in points.")
                 headlength = d.pop('headlength', 12)
 
-                to_style = self.figure.dpi / (72 * ms)
-
-                stylekw = dict(head_length=headlength * to_style,
-                               head_width=headwidth * to_style,
-                               tail_width=width * to_style)
+                # NB: ms is in pts
+                stylekw = dict(head_length=headlength / ms,
+                               head_width=headwidth / ms,
+                               tail_width=width / ms)
 
                 self.arrow_patch.set_arrowstyle('simple', **stylekw)
 
@@ -2257,13 +2223,8 @@ class Annotation(Text, _AnnotationBase):
                 xpos = ((l, 0), (xc, 0.5), (r, 1))
                 ypos = ((b, 0), (yc, 0.5), (t, 1))
 
-                dsu = [(abs(val[0] - x0), val) for val in xpos]
-                dsu.sort()
-                _, (x, relposx) = dsu[0]
-
-                dsu = [(abs(val[0] - y0), val) for val in ypos]
-                dsu.sort()
-                _, (y, relposy) = dsu[0]
+                _, (x, relposx) = min((abs(val[0] - x0), val) for val in xpos)
+                _, (y, relposy) = min((abs(val[0] - y0), val) for val in ypos)
 
                 self._arrow_relpos = (relposx, relposy)
 
@@ -2282,7 +2243,7 @@ class Annotation(Text, _AnnotationBase):
 
             # The arrow will be drawn from (ox0, oy0) to (ox1,
             # oy1). It will be first clipped by patchA and patchB.
-            # Then it will be shrunk by shirnkA and shrinkB
+            # Then it will be shrunk by shrinkA and shrinkB
             # (in points). If patch A is not set, self.bbox_patch
             # is used.
 
@@ -2295,7 +2256,7 @@ class Annotation(Text, _AnnotationBase):
                     self.arrow_patch.set_patchA(self._bbox_patch)
                 else:
                     pad = renderer.points_to_pixels(4)
-                    if self.get_text().strip() == "":
+                    if self.get_text() == "":
                         self.arrow_patch.set_patchA(None)
                         return
 
@@ -2309,12 +2270,12 @@ class Annotation(Text, _AnnotationBase):
                                   width=w,
                                   height=h,
                                   )
-                    r.set_transform(mtransforms.IdentityTransform())
+                    r.set_transform(IdentityTransform())
                     r.set_clip_on(False)
 
                     self.arrow_patch.set_patchA(r)
 
-    @allow_rasterization
+    @artist.allow_rasterization
     def draw(self, renderer):
         """
         Draw the :class:`Annotation` object to the given *renderer*.
