@@ -1,15 +1,22 @@
 """
-Show how to modify the coordinate formatter to report the image "z"
+==================================
+Modifying the coordinate formatter
+==================================
+
+Modify the coordinate formatter to report the image "z"
 value of the nearest pixel given x and y
 """
 import numpy as np
 import matplotlib.pyplot as plt
-import matplotlib.cm as cm
+
+# Fixing random state for reproducibility
+np.random.seed(19680801)
+
 
 X = 10*np.random.rand(5, 3)
 
 fig, ax = plt.subplots()
-ax.imshow(X, cmap=cm.jet, interpolation='nearest')
+ax.imshow(X, interpolation='nearest')
 
 numrows, numcols = X.shape
 
