@@ -1,13 +1,19 @@
-#!/usr/local/bin/python3.6
-# A matplotlib based game of Pong illustrating one way to write interactive
-# animation which are easily ported to multiple backends
-# pipong.py was written by Paul Ivanov <http://pirsquared.org>
+"""
+======
+Pipong
+======
+
+A Matplotlib based game of Pong illustrating one way to write interactive
+animation which are easily ported to multiple backends
+pipong.py was written by Paul Ivanov <http://pirsquared.org>
+"""
 
 from __future__ import print_function
 
 import numpy as np
 import matplotlib.pyplot as plt
 from numpy.random import randn, randint
+from matplotlib.font_manager import FontProperties
 
 instructions = """
 Player A:       Player B:
@@ -80,14 +86,11 @@ class Puck(object):
                 self.vy *= 1.2 * pad.signy
         fudge = .001
         # probably cleaner with something like...
-        #if not self.field.contains(self.x, self.y):
         if self.x < fudge:
-            #print("player A loses")
             pads[1].score += 1
             self._reset(pads[0])
             return True
         if self.x > 7 - fudge:
-            #print("player B loses")
             pads[0].score += 1
             self._reset(pads[1])
             return True
@@ -122,20 +125,42 @@ class Game(object):
     def __init__(self, ax):
         # create the initial line
         self.ax = ax
-        padAx = padBx = .50
+        ax.set_ylim([-1, 1])
+        ax.set_xlim([0, 7])
+        padAx = 0
+        padBx = .50
         padAy = padBy = .30
         padBx += 6.3
-        pA, = self.ax.barh(padAy, .2, height=.3, color='k', alpha=.5, edgecolor='b', lw=2, label="Player B", animated=True)
-        pB, = self.ax.barh(padBy, .2, height=.3, left=padBx, color='k', alpha=.5, edgecolor='r', lw=2, label="Player A", animated=True)
+
+        # pads
+        pA, = self.ax.barh(padAy, .2,
+                           height=.3, color='k', alpha=.5, edgecolor='b',
+                           lw=2, label="Player B",
+                           animated=True)
+        pB, = self.ax.barh(padBy, .2,
+                           height=.3, left=padBx, color='k', alpha=.5,
+                           edgecolor='r', lw=2, label="Player A",
+                           animated=True)
 
         # distractors
         self.x = np.arange(0, 2.22*np.pi, 0.01)
-        self.line, = self.ax.plot(self.x, np.sin(self.x), "r", animated=True, lw=4)
-        self.line2, = self.ax.plot(self.x, np.cos(self.x), "g", animated=True, lw=4)
-        self.line3, = self.ax.plot(self.x, np.cos(self.x), "g", animated=True, lw=4)
-        self.line4, = self.ax.plot(self.x, np.cos(self.x), "r", animated=True, lw=4)
-        self.centerline, = self.ax.plot([3.5, 3.5], [1, -1], 'k', alpha=.5, animated=True, lw=8)
-        self.puckdisp = self.ax.scatter([1], [1], label='_nolegend_', s=200, c='g', alpha=.9, animated=True)
+        self.line, = self.ax.plot(self.x, np.sin(self.x), "r",
+                                  animated=True, lw=4)
+        self.line2, = self.ax.plot(self.x, np.cos(self.x), "g",
+                                   animated=True, lw=4)
+        self.line3, = self.ax.plot(self.x, np.cos(self.x), "g",
+                                   animated=True, lw=4)
+        self.line4, = self.ax.plot(self.x, np.cos(self.x), "r",
+                                   animated=True, lw=4)
+
+        # center line
+        self.centerline, = self.ax.plot([3.5, 3.5], [1, -1], 'k',
+                                        alpha=.5, animated=True, lw=8)
+
+        # puck (s)
+        self.puckdisp = self.ax.scatter([1], [1], label='_nolegend_',
+                                        s=200, c='g',
+                                        alpha=.9, animated=True)
 
         self.canvas = self.ax.figure.canvas
         self.background = None
@@ -146,7 +171,7 @@ class Game(object):
         self.inst = True    # show instructions from the beginning
         self.background = None
         self.pads = []
-        self.pads.append(Pad(pA, 0, padAy))
+        self.pads.append(Pad(pA, padAx, padAy))
         self.pads.append(Pad(pB, padBx, padBy, 'r'))
         self.pucks = []
         self.i = self.ax.annotate(instructions, (.5, 0.5),
@@ -154,7 +179,8 @@ class Game(object):
                                   verticalalignment='center',
                                   horizontalalignment='center',
                                   multialignment='left',
-                                  textcoords='axes fraction', animated=True)
+                                  textcoords='axes fraction',
+                                  animated=False)
         self.canvas.mpl_connect('key_press_event', self.key_press)
 
     def draw(self, evt):
@@ -176,10 +202,6 @@ class Game(object):
             draw_artist(self.line3)
             draw_artist(self.line4)
 
-        # show the instructions - this is very slow
-        if self.inst:
-            self.ax.draw_artist(self.i)
-
         # pucks and pads
         if self.on:
             self.ax.draw_artist(self.centerline)
@@ -191,23 +213,24 @@ class Game(object):
             for puck in self.pucks:
                 if puck.update(self.pads):
                     # we only get here if someone scored
-                    self.pads[0].disp.set_label("   " + str(self.pads[0].score))
-                    self.pads[1].disp.set_label("   " + str(self.pads[1].score))
-                    self.ax.legend(loc='center')
-                    self.leg = self.ax.get_legend()
-                    #self.leg.draw_frame(False) #don't draw the legend border
-                    self.leg.get_frame().set_alpha(.2)
-                    plt.setp(self.leg.get_texts(), fontweight='bold', fontsize='xx-large')
-                    self.leg.get_frame().set_facecolor('0.2')
+                    self.pads[0].disp.set_label(
+                        "   " + str(self.pads[0].score))
+                    self.pads[1].disp.set_label(
+                        "   " + str(self.pads[1].score))
+                    self.ax.legend(loc='center', framealpha=.2,
+                                   facecolor='0.5',
+                                   prop=FontProperties(size='xx-large',
+                                                       weight='bold'))
+
                     self.background = None
-                    self.ax.figure.canvas.draw()
+                    self.ax.figure.canvas.draw_idle()
                     return True
-                puck.disp.set_offsets([puck.x, puck.y])
+                puck.disp.set_offsets([[puck.x, puck.y]])
                 self.ax.draw_artist(puck.disp)
 
         # just redraw the axes rectangle
         self.canvas.blit(self.ax.bbox)
-
+        self.canvas.flush_events()
         if self.cnt == 50000:
             # just so we don't get carried away
             print("...and you've been playing for too long!!!")
@@ -241,7 +264,9 @@ class Game(object):
                 self.pads[1].y = -1
 
         if event.key == 'a':
-            self.pucks.append(Puck(self.puckdisp, self.pads[randint(2)], self.ax.bbox))
+            self.pucks.append(Puck(self.puckdisp,
+                                   self.pads[randint(2)],
+                                   self.ax.bbox))
         if event.key == 'A' and len(self.pucks):
             self.pucks.pop()
         if event.key == ' ' and len(self.pucks):
@@ -257,10 +282,11 @@ class Game(object):
             self.distract = not self.distract
 
         if event.key == 'g':
-            #self.ax.clear()
             self.on = not self.on
         if event.key == 't':
             self.inst = not self.inst
-            self.i.set_visible(self.i.get_visible())
+            self.i.set_visible(not self.i.get_visible())
+            self.background = None
+            self.canvas.draw_idle()
         if event.key == 'q':
             plt.close()
